------------------------------
--      Are you local?      --
------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Necronomicon")
local BS = AceLibrary("Babble-Spell-2.2")

---------------------------------
--      Addon Declaration      --
---------------------------------

Necronomicon = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceModuleCore-2.0", "AceConsole-2.0", "AceDB-2.0")

Necronomicon.revision = tonumber(string.sub("$Revision: 26487 $", 12, -3))

Necronomicon:SetModuleMixins("AceEvent-2.0")

Necronomicon.defaults = {}

Necronomicon.consoleOptions = {
	type = "group",
	handler = Necronomicon,
	args = {},
}

Necronomicon:RegisterDB("NecronomiconDB", "NecronomiconDBPerChar")
Necronomicon:RegisterDefaults("profile", Necronomicon.defaults)
Necronomicon:RegisterChatCommand({ "/Necronomicon" }, Necronomicon.consoleOptions, "NECRONOMICON")

------------------------------
--      Initialization      --
------------------------------

function Necronomicon:OnInitialize()
	self.spelllist = {}
	self.itemlist = {}
	if not self.version then self.version = GetAddOnMetadata("Necronomicon", "Version") end
	local rev, name = self.revision, nil
	for name, module in self:IterateModules() do
		rev = math.max(rev, module.revision)
	end
	self.version = (self.version or "1.0") .. " |cffff8888r" .. rev .. "|r"
end

function Necronomicon:OnEnable()
	local class, enclass = UnitClass("player")
	if enclass == "WARLOCK" then
	
		local name, module = nil, nil
		for name, module in self:IterateModules() do
			if not self:IsModuleActive(module) then
				self:ToggleModuleActive(module, true)
			end
		end

		-- first learn all spells. Do not change order.
		self:RegisterEvent("LEARNED_SPELL_IN_TAB")
		self:LEARNED_SPELL_IN_TAB()
		-- scan bags. Do not change order.
		self:RegisterEvent("BAG_UPDATE")
		self:BAG_UPDATE()
	else
		self:ToggleActive(false)
	end
end

function Necronomicon:OnDisable()
	local name, module = nil, nil
	for name, module in self:IterateModules() do
		self:ToggleModuleActive(module, false)
	end
end

-------------------------------
--      Module Handling      --
-------------------------------

function Necronomicon:RegisterModule(name, module)
	if module:IsRegistered() then
		error(string.format("%q is already registered.", name))
		return
	end

	if module.db and module.RegisterDefaults and type(module.RegisterDefaults) == "function" then
		module:RegisterDefaults("profile", module.defaults or {})
	else
		self:RegisterDefaults(name, "profile", module.defaults or {})
	end

	if not module.db then module.db = self:AcquireDBNamespace(name) end

	if module.consoleOptions then
		local m = module.consoleCmd or name
		-- if the consoleoption already exists we merge in the data otherwise we create a new option
		if self.consoleOptions.args[m] then
			for k,v in pairs(module.consoleOptions.args) do self.consoleOptions.args[m].args[k] = v end
		else
			self.consoleOptions.args[m] = module.consoleOptions
		end
	end

	module.registered = true
	if module.OnRegister and type(module.OnRegister) == "function" then
		module:OnRegister()
	end
end

---------------------------------
--       Event Handlers        --
---------------------------------

function Necronomicon:BAG_UPDATE()
	self:ScheduleEvent("NecronomiconBag", self.ScanBags, 0.5, self )
end

function Necronomicon:LEARNED_SPELL_IN_TAB()
	local i = 1
	local enspell

	self:TriggerEvent("Necronomicon_ClearSpells")

	while true do
		local spell, subspell = GetSpellName(i, BOOKTYPE_SPELL)
		if not spell then
			do break end
		end
		enspell = BS:GetReverseTranslation( spell )
		if enspell and self.spellist[enspell] then
			self:TriggerEvent("Necronomicon_HasSpell", enspell, spell, subspell, i )
		end
		i = i + 1
	end
end

function Necronomicon:ScanBags()
	for bag = 0, 4, 1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do
			local _,_,itemid = string.find(GetContainerItemLink(bag,slot) or "", "item:(%d+):%d+:%d+:%d+")
			itemid = tonumber(itemid)
			if itemid and self.itemlist[itemid] then
				self:TriggerEvent("Necronomicon_BagUpdate", itemid, bag, slot )
			end
		end
	end
end

