assert( Necronomicon, "Necronomicon not found!")

----------------------------------
--        Are you local?        --
----------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Necronomicon")

local soulshardid = 6265
local soulshardcount = 0
local soulshardicon = "Interface\\Icons\\INV_Misc_Gem_Amethyst_02"

----------------------------------
--      Module Declaration      --
----------------------------------

NecronomiconSoulShard = Necronomicon:NewModule("SoulShard")

NecronomiconSoulShard.revision = tonumber(string.sub("$Revision: 26487 $", 12, -3))

NecronomiconSoulShard.defaults = {
	low = 5,
	destroy = true,
	posx = 200,
	posy = 200,
}

----------------------------------
--     Module Intialization     --
----------------------------------

function NecronomiconSoulShard:OnRegister()
	self:RegisterSpell("Drain Soul")
end

function NecronomiconSoulShard:OnEnable()
	self:SetupButton()

	self:RegisterEvent("Necronomicon_ClearSpells")
	self:RegisterEvent("Necronomicon_HasSpell")

	-- register the normal BAG_UPDATE for soulshards, it's not like the stones
	self:RegisterEvent("BAG_UPDATE")
	self:BAG_UPDATE()
end

----------------------------------
--        Event Handlers        --
----------------------------------

function NecronomiconSoulShard:BAG_UPDATE()
	self:ScheduleEvent("NecronomiconSoulShard", self.CountShards, 0.5, self)
end

function NecronomiconSoulShard:CountShards()
	local oldshards = soulshardcount
	soulshardcount = GetItemCount( soulshardid )
	if soulshardcount ~= oldshards then
		self:DestroyShards()
		self:Warn( "Soulshards: "..soulshardcount )
		if soulshardcount <= self.db.profile.low then
			self:Warn( string.format( L["Low on Shards! %d left!"], soulshardcount ) )
		end
		self.button.text:SetText( soulshardcount )
	end
end

function NecronomiconSoulShard:Necronomicon_ClearSpells()
	self.button:SetAttribute("type", "")
	self.button:SetAttribute("spell", "")
	self.button:Hide()
end

function NecronomiconSoulShard:Necronomicon_HasSpell( enspell, spell, subspell, spellid )
	if not enspell then return end
	if enspell == "Drain Soul" then
		self.button:SetAttribute("type", "spell")
		self.button:SetAttribute("spell", GetSpellName( spellid, BOOKTYPE_SPELL) )
		self.button:Show()
	end
end

----------------------------------
--       Utility Functions      --
----------------------------------

function NecronomiconSoulShard:DestroyShards()
	-- destroy shards from normal bags

	if not self.db.profile.destroy then return end

	for bag = 0, 4, 1 do
		local cname = GetBagName(bag)
		if cname ~= L["Soul Pouch"] and cname ~= L["Felcloth Bag"] and cname ~= L["Core Felcloth Bag"] then
			for slot = 1, GetContainerNumSlots(bag), 1 do
				local _,_,itemid = string.find(GetContainerItemLink(bag,slot) or "", "item:(%d+):%d+:%d+:%d+")
				itemid = tonumber(itemid)
				if itemid and itemid == soulshardid then
					PickupContainerItem(bag, slot)
					if( CursorHasItem()) then
						DeleteCursorItem()
					end
				end
			end
		end
	end

	soulshardcount = GetItemCount( soulshardid )
end

----------------------------------
--         GUI Functions        --
----------------------------------

function NecronomiconSoulShard:SetupButton()
	self:CreateButton()

	self.button:SetNormalTexture( soulshardicon )

	self.button.text = self.button:CreateFontString(nil, "OVERLAY")
	self.button.text:SetFontObject(GameFontNormalSmall)
	self.button.text:ClearAllPoints()
	self.button.text:SetTextColor(1, 1, 1, 1) 
	self.button.text:SetWidth(32)
	self.button.text:SetHeight(32)
	self.button.text:SetPoint("TOPLEFT", self.button, "TOPLEFT")
	self.button.text:SetJustifyH("CENTER")
	self.button.text:SetJustifyV("MIDDLE")
	self.button.text:SetText(0)
end
